import 'package:flutter/material.dart';
import 'app_screen.dart';
import 'app_screen_route_path.dart';
import 'app_screens.dart';

class AppScreenRouteInformationParser
    extends RouteInformationParser<AppScreenRoutePath> {
  @override
  Future<AppScreenRoutePath> parseRouteInformation(
      RouteInformation routeInformation) async {
    final uri = Uri.parse(routeInformation.location);
    // Handle '/'
    if (uri.pathSegments.length == 0) {
      return AppScreenRoutePath.home();
    }

    print("Complete URL: $uri");

    if (uri.pathSegments.length == 1 || uri.pathSegments.length == 2) {
      var name = uri.path.substring(1, uri.path.length);

      print("name placed: " + name);

      if (name == null) {
        return AppScreenRoutePath.unknown();
      } else {
        AppScreen appScreen = appScreens.singleWhere(
            (currentAppScreen) => currentAppScreen.name == name, orElse: () {
          return null;
        });

        if (appScreen == null) {
          return AppScreenRoutePath.unknown();
        } else {
          return AppScreenRoutePath.details(name);
        }
      }
    }

    // Handle unknown routes
    return AppScreenRoutePath.unknown();
  }

  @override
  RouteInformation restoreRouteInformation(AppScreenRoutePath path) {
    if (path.isUnknown) {
      return RouteInformation(location: '/404');
    }
    if (path.isHomePage) {
      return RouteInformation(location: '/');
    }
    if (path.isDetailsPage) {
      return RouteInformation(location: '/${path.name}');
    }
    return null;
  }
}
